using System;
using System.IO;

namespace AdrianKousz.Util.Logging
{
	public class SimpleLogger : Logger
	{
		public string DateFormat = "yyyy-MM-dd HH:mm:ss.fff zzzz";
		public bool Colored = true;
		public bool Dated = true;

		public string[] Letters = { "X","A","C","E","W","N","I","D" };
		public string[] Colors = {
			"\u001b[37;45;1;5m",
			"\u001b[37;41;1m",
			"\u001b[37;43;1m",
			"\u001b[31m",
			"\u001b[33m",
			"\u001b[34m",
			"\u001b[36m",
			"\u001b[37m",
		};

		private readonly TextWriter writer;
		private readonly Object sync;

		public SimpleLogger(TextWriter writer)
		{
			this.writer = writer;
			sync = new object();
		}

		public void Log(LogMsg msg)
		{
			var template = "{0}{1}{2}/{3}:{4} {5}{6}";
			var colorStart = "";
			var colorEnd = "";
			var date = "";
			var letter = "";

			if (Colored) {
				colorStart = Colors[msg.Level];
				colorEnd = "\u001b[0m";
			}

			if (Dated) {
				date = "{{0:{0}}} ";
				date = string.Format(date, DateFormat);
				date = string.Format(date, msg.Timestamp);
			}

			letter = Letters[msg.Level];

			var line = string.Format(template, date, colorStart, letter, msg.Tag, colorEnd, msg.Message, Environment.NewLine);

			lock (sync) {
				writer.Write(line);
				writer.Flush();
			}
		}
	}
}
