﻿using System;
using System.Collections.Generic;
using AdrianKousz.Util.Logging;

namespace AdrianKousz.Util
{
	public static class Log
	{
		/** Emergency: Deadly */
		public const int EMERG   = 0;
		/** Alert: The application can not continue */
		public const int ALERT   = 1;
		/** Critical: The application will likely produce invalid results */
		public const int CRIT    = 2;
		/** Error: Parts of the application are unstable */
		public const int ERR     = 3;
		/** Warning: There is an invalid condition, but operation can continue  */
		public const int WARNING = 4;
		/** Notice: Normal, but significant conditions */
		public const int NOTICE  = 5;
		/** Informational: Normal operational messages */
		public const int INFO    = 6;
		/** Debug: Information generally only useful for debugging */
		public const int DEBUG   = 7;

		public static bool Debug = Helper.IsDebug();

		private static readonly List<Logger> loggers = new List<Logger>();

		public static void log(int level, string tag, string message)
		{
			if (level >= DEBUG && !Debug) return;
			LogMsg logmsg = new LogMsg(level, tag, message);
			if (loggers.Count == 0) {
				Helper.GetDefault().Log(logmsg);
				return;
			}
			foreach (var logger in loggers) {
				logger.Log(logmsg);
			}
		}

		public static void log(Exception e)
		{
			Helper.Log(e);
		}

		public static void AddLogger(Logger logger)
		{
			loggers.Add(logger);
		}

		public static void SetUncaughtExceptionHandler()
		{
			Helper.SetHandler();
		}

		#region Shortcut Methods

		public static void x(string tag, string msg) { log(EMERG,   tag, msg); }
		public static void a(string tag, string msg) { log(ALERT,   tag, msg); }
		public static void c(string tag, string msg) { log(CRIT,    tag, msg); }
		public static void e(string tag, string msg) { log(ERR,     tag, msg); }
		public static void w(string tag, string msg) { log(WARNING, tag, msg); }
		public static void n(string tag, string msg) { log(NOTICE,  tag, msg); }
		public static void i(string tag, string msg) { log(INFO,    tag, msg); }
		public static void d(string tag, string msg) { if (!Debug) return; log(DEBUG, tag, msg); }

		public static void x(string tag, string fmt, params object[] arg) { log(EMERG,   tag, string.Format(fmt, arg)); }
		public static void a(string tag, string fmt, params object[] arg) { log(ALERT,   tag, string.Format(fmt, arg)); }
		public static void c(string tag, string fmt, params object[] arg) { log(CRIT,    tag, string.Format(fmt, arg)); }
		public static void e(string tag, string fmt, params object[] arg) { log(ERR,     tag, string.Format(fmt, arg)); }
		public static void w(string tag, string fmt, params object[] arg) { log(WARNING, tag, string.Format(fmt, arg)); }
		public static void n(string tag, string fmt, params object[] arg) { log(NOTICE,  tag, string.Format(fmt, arg)); }
		public static void i(string tag, string fmt, params object[] arg) { log(INFO,    tag, string.Format(fmt, arg)); }
		public static void d(string tag, string fmt, params object[] arg) { if (!Debug) return; log(DEBUG, tag, string.Format(fmt, arg)); }

		#endregion
	}
}

